@@ -80,6 +80,7 @@ struct huawei_data {
 	gboolean ndis;
 	guint sim_poll_timeout;
 	guint sim_poll_count;
+	guint reopen_timeout;
 };
 
 #define MAX_SIM_POLL_COUNT 5
@@ -107,6 +108,11 @@ static void huawei_remove(struct ofono_modem *modem)
 
 	DBG("%p", modem);
 
+	if (data->reopen_timeout > 0) {
+		g_source_remove(data->reopen_timeout);
+		data->reopen_timeout = 0;
+	}
+
 	ofono_modem_set_data(modem, NULL);
 
 	if (data->modem)
@@ -217,7 +223,8 @@ static gboolean notify_sim_state(struct ofono_modem *modem,
 		 */
 		ofono_modem_set_powered(modem, TRUE);
 
-		g_at_chat_send(data->pcui, "AT+CFUN=5", none_prefix,
+		if (ofono_modem_get_online(modem) == FALSE)
+			g_at_chat_send(data->pcui, "AT+CFUN=5", none_prefix,
 				cfun_offline, modem, NULL);
 
 		return FALSE;
@@ -464,19 +471,44 @@ static GAtChat *open_device(struct ofono_modem *modem,
 	return chat;
 }
 
+static void huawei_disconnect(gpointer user_data);
+
+static gboolean reopen_callback(gpointer user_data)
+{
+	struct ofono_modem *modem = user_data;
+	struct huawei_data *data = ofono_modem_get_data(modem);
+
+	DBG("%p", modem);
+
+	huawei_disconnect(user_data);
+
+	data->reopen_timeout = 0;
+
+	return FALSE;
+}
+
 static void huawei_disconnect(gpointer user_data)
 {
 	struct ofono_modem *modem = user_data;
 	struct huawei_data *data = ofono_modem_get_data(modem);
 
-	DBG("data->gc %p", data->gc);
+	DBG("%p, data->gc %p", modem, data->gc);
 
 	g_at_chat_unref(data->modem);
 	data->modem = NULL;
 
 	data->modem = open_device(modem, "Modem", "Modem: ");
-	if (data->modem == NULL)
+	/* retry once if failed */
+	if (data->modem == NULL) {
+		if (data->reopen_timeout > 0)
+			return;
+
+		data->reopen_timeout = g_timeout_add_seconds(1,
+				reopen_callback, modem);
+
+		ofono_debug("opening modem port failed, retrying...");
 		return;
+	}
 
 	g_at_chat_set_disconnect_function(data->modem,
 						huawei_disconnect, modem);
@@ -558,6 +590,11 @@ static int huawei_disable(struct ofono_modem *modem)
 
 	DBG("%p", modem);
 
+	if (data->reopen_timeout > 0) {
+		g_source_remove(data->reopen_timeout);
+		data->reopen_timeout = 0;
+	}
+
 	if (data->sim_poll_timeout > 0) {
 		g_source_remove(data->sim_poll_timeout);
 		data->sim_poll_timeout = 0;
@@ -662,7 +699,6 @@ static void huawei_post_sim(struct ofono_modem *modem)
 static void huawei_post_online(struct ofono_modem *modem)
 {
 	struct huawei_data *data = ofono_modem_get_data(modem);
-	struct ofono_netreg *netreg;
 	struct ofono_message_waiting *mw;
 
 	if (data->sim_state != HUAWEI_SIM_STATE_VALID &&
@@ -673,8 +709,7 @@ static void huawei_post_online(struct ofono_modem *modem)
 		return;
 	}
 
-	netreg = ofono_netreg_create(modem, OFONO_VENDOR_HUAWEI, "atmodem",
-								data->pcui);
+	ofono_netreg_create(modem, OFONO_VENDOR_HUAWEI, "atmodem", data->pcui);
 
 	ofono_cbs_create(modem, OFONO_VENDOR_QUALCOMM_MSM,
 						"atmodem", data->pcui);
@@ -689,7 +724,7 @@ static void huawei_post_online(struct ofono_modem *modem)
 		if (data->ndis == TRUE)
 			data->gc = ofono_gprs_context_create(modem, 0,
 						"huaweimodem", data->pcui);
-		else
+		else if (data->modem != NULL)
 			data->gc = ofono_gprs_context_create(modem, 0,
 						"atmodem", data->modem);
 
