@@ -37,11 +37,15 @@
 #include "common.h"
 #include "simutil.h"
 #include "smsutil.h"
+#include "storage.h"
 
 #define MAX_VOICE_CALLS 16
 
 #define VOICECALL_FLAG_SIM_ECC_READY 0x1
 
+#define SETTINGS_STORE "voicecall"
+#define SETTINGS_GROUP "Settings"
+
 GSList *g_drivers = NULL;
 
 struct multi_release {
@@ -71,6 +75,10 @@ struct ofono_voicecall {
 	GQueue *toneq;
 	guint tone_source;
 	unsigned int hfp_watch;
+	GKeyFile *settings;
+	char *imsi;
+	struct ofono_emulator *pending_em;
+	char *em_atd_number;
 };
 
 struct voicecall {
@@ -106,6 +114,11 @@ struct tone_queue_entry {
 	int id;
 };
 
+struct emulator_status {
+	struct ofono_voicecall *vc;
+	int status;
+};
+
 static const char *default_en_list[] = { "911", "112", NULL };
 static const char *default_en_list_no_sim[] = { "119", "118", "999", "110",
 						"08", "000", NULL };
@@ -698,31 +711,41 @@ static void voicecall_emit_multiparty(struct voicecall *call, gboolean mpty)
 static void emulator_call_status_cb(struct ofono_atom *atom, void *data)
 {
 	struct ofono_emulator *em = __ofono_atom_get_data(atom);
+	struct emulator_status *s = data;
 
-	ofono_emulator_set_indicator(em, OFONO_EMULATOR_IND_CALL,
-						GPOINTER_TO_INT(data));
+	if (em == s->vc->pending_em)
+		return;
+
+	ofono_emulator_set_indicator(em, OFONO_EMULATOR_IND_CALL, s->status);
 }
 
 static void emulator_callsetup_status_cb(struct ofono_atom *atom, void *data)
 {
 	struct ofono_emulator *em = __ofono_atom_get_data(atom);
+	struct emulator_status *s = data;
+
+	if (em == s->vc->pending_em)
+		return;
 
 	ofono_emulator_set_indicator(em, OFONO_EMULATOR_IND_CALLSETUP,
-						GPOINTER_TO_INT(data));
+					s->status);
 }
 
 static void emulator_callheld_status_cb(struct ofono_atom *atom, void *data)
 {
 	struct ofono_emulator *em = __ofono_atom_get_data(atom);
+	struct emulator_status *s = data;
+
+	if (em == s->vc->pending_em)
+		return;
 
 	ofono_emulator_set_indicator(em, OFONO_EMULATOR_IND_CALLHELD,
-						GPOINTER_TO_INT(data));
+					s->status);
 }
 
 static void notify_emulator_call_status(struct ofono_voicecall *vc)
 {
 	struct ofono_modem *modem = __ofono_atom_get_modem(vc->atom);
-	int status;
 	gboolean call = FALSE;
 	gboolean held = FALSE;
 	gboolean incoming = FALSE;
@@ -731,6 +754,9 @@ static void notify_emulator_call_status(struct ofono_voicecall *vc)
 	gboolean waiting = FALSE;
 	GSList *l;
 	struct voicecall *v;
+	struct emulator_status data;
+
+	data.vc = vc;
 
 	for (l = vc->call_list; l; l = l->next) {
 		v = l->data;
@@ -762,40 +788,39 @@ static void notify_emulator_call_status(struct ofono_voicecall *vc)
 		}
 	}
 
-	status = call || held ? OFONO_EMULATOR_CALL_ACTIVE :
+	data.status = call || held ? OFONO_EMULATOR_CALL_ACTIVE :
 					OFONO_EMULATOR_CALL_INACTIVE;
 
 	__ofono_modem_foreach_registered_atom(modem,
 						OFONO_ATOM_TYPE_EMULATOR_HFP,
-						emulator_call_status_cb,
-						GINT_TO_POINTER(status));
+						emulator_call_status_cb, &data);
 
 	if (incoming)
-		status = OFONO_EMULATOR_CALLSETUP_INCOMING;
+		data.status = OFONO_EMULATOR_CALLSETUP_INCOMING;
 	else if (dialing)
-		status = OFONO_EMULATOR_CALLSETUP_OUTGOING;
+		data.status = OFONO_EMULATOR_CALLSETUP_OUTGOING;
 	else if (alerting)
-		status = OFONO_EMULATOR_CALLSETUP_ALERTING;
+		data.status = OFONO_EMULATOR_CALLSETUP_ALERTING;
 	else if (waiting)
-		status = OFONO_EMULATOR_CALLSETUP_INCOMING;
+		data.status = OFONO_EMULATOR_CALLSETUP_INCOMING;
 	else
-		status = OFONO_EMULATOR_CALLSETUP_INACTIVE;
+		data.status = OFONO_EMULATOR_CALLSETUP_INACTIVE;
 
 	__ofono_modem_foreach_registered_atom(modem,
 						OFONO_ATOM_TYPE_EMULATOR_HFP,
 						emulator_callsetup_status_cb,
-						GINT_TO_POINTER(status));
+						&data);
 
 	if (held)
-		status = call ? OFONO_EMULATOR_CALLHELD_MULTIPLE :
+		data.status = call ? OFONO_EMULATOR_CALLHELD_MULTIPLE :
 					OFONO_EMULATOR_CALLHELD_ON_HOLD;
 	else
-		status = OFONO_EMULATOR_CALLHELD_NONE;
+		data.status = OFONO_EMULATOR_CALLHELD_NONE;
 
 	__ofono_modem_foreach_registered_atom(modem,
 						OFONO_ATOM_TYPE_EMULATOR_HFP,
 						emulator_callheld_status_cb,
-						GINT_TO_POINTER(status));
+						&data);
 }
 
 static void voicecall_set_call_status(struct voicecall *call, int status)
@@ -1437,59 +1462,93 @@ static void manager_dial_callback(const struct ofono_error *error, void *data)
 		voicecalls_emit_call_added(vc, v);
 }
 
+static int voicecall_dial(struct ofono_voicecall *vc, const char *number,
+					enum ofono_clir_option clir,
+					ofono_voicecall_cb_t cb, void *data)
+{
+	struct ofono_modem *modem = __ofono_atom_get_modem(vc->atom);
+	struct ofono_phone_number ph;
+
+	if (g_slist_length(vc->call_list) >= MAX_VOICE_CALLS)
+		return -EPERM;
+
+	if (!valid_long_phone_number_format(number))
+		return -EINVAL;
+
+	if (ofono_modem_get_online(modem) == FALSE)
+		return -ENETDOWN;
+
+	if (vc->driver->dial == NULL)
+		return -ENOTSUP;
+
+	if (voicecalls_have_incoming(vc))
+		return -EBUSY;
+
+	/* We can't have two dialing/alerting calls, reject outright */
+	if (voicecalls_num_connecting(vc) > 0)
+		return -EBUSY;
+
+	if (voicecalls_have_active(vc) && voicecalls_have_held(vc))
+		return -EBUSY;
+
+	if (is_emergency_number(vc, number) == TRUE)
+		__ofono_modem_inc_emergency_mode(modem);
+
+	string_to_phone_number(number, &ph);
+
+	vc->driver->dial(vc, &ph, clir, cb, vc);
+
+	if (vc->settings) {
+		g_key_file_set_string(vc->settings, SETTINGS_GROUP,
+					"Number", number);
+		storage_sync(vc->imsi, SETTINGS_STORE, vc->settings);
+	}
+
+	return 0;
+}
+
 static DBusMessage *manager_dial(DBusConnection *conn,
 					DBusMessage *msg, void *data)
 {
 	struct ofono_voicecall *vc = data;
-	struct ofono_modem *modem = __ofono_atom_get_modem(vc->atom);
 	const char *number;
-	struct ofono_phone_number ph;
 	const char *clirstr;
 	enum ofono_clir_option clir;
+	int err;
 
-	if (vc->pending)
+	if (vc->pending || vc->dial_req || vc->pending_em)
 		return __ofono_error_busy(msg);
 
-	if (g_slist_length(vc->call_list) >= MAX_VOICE_CALLS)
-		return __ofono_error_failed(msg);
-
 	if (dbus_message_get_args(msg, NULL, DBUS_TYPE_STRING, &number,
 					DBUS_TYPE_STRING, &clirstr,
 					DBUS_TYPE_INVALID) == FALSE)
 		return __ofono_error_invalid_args(msg);
 
-	if (!valid_long_phone_number_format(number))
-		return __ofono_error_invalid_format(msg);
-
 	if (clir_string_to_clir(clirstr, &clir) == FALSE)
 		return __ofono_error_invalid_format(msg);
 
-	if (ofono_modem_get_online(modem) == FALSE)
-		return __ofono_error_not_available(msg);
-
-	if (vc->driver->dial == NULL)
-		return __ofono_error_not_implemented(msg);
-
-	if (voicecalls_have_incoming(vc))
-		return __ofono_error_failed(msg);
-
-	/* We can't have two dialing/alerting calls, reject outright */
-	if (voicecalls_num_connecting(vc) > 0)
-		return __ofono_error_failed(msg);
-
-	if (voicecalls_have_active(vc) && voicecalls_have_held(vc))
-		return __ofono_error_failed(msg);
-
-	if (is_emergency_number(vc, number) == TRUE)
-		__ofono_modem_inc_emergency_mode(modem);
-
 	vc->pending = dbus_message_ref(msg);
 
-	string_to_phone_number(number, &ph);
+	err = voicecall_dial(vc, number, clir, manager_dial_callback, vc);
 
-	vc->driver->dial(vc, &ph, clir, manager_dial_callback, vc);
+	if (err >= 0)
+		return NULL;
 
-	return NULL;
+	vc->pending = NULL;
+	dbus_message_unref(msg);
+
+	switch (err) {
+	case -EINVAL:
+		return __ofono_error_invalid_format(msg);
+
+	case -ENETDOWN:
+		return __ofono_error_not_available(msg);
+
+	case -ENOTSUP:
+		return __ofono_error_not_implemented(msg);
+	}
+
+	return __ofono_error_failed(msg);
 }
 
 static DBusMessage *manager_transfer(DBusConnection *conn,
@@ -2417,10 +2476,53 @@ static void emulator_hfp_unregister(struct ofono_atom *atom)
 						OFONO_ATOM_TYPE_EMULATOR_HFP,
 						emulator_remove_handler,
 						"+CLCC");
+	__ofono_modem_foreach_registered_atom(modem,
+						OFONO_ATOM_TYPE_EMULATOR_HFP,
+						emulator_remove_handler,
+						"+CHLD");
+	__ofono_modem_foreach_registered_atom(modem,
+						OFONO_ATOM_TYPE_EMULATOR_HFP,
+						emulator_remove_handler,
+						"+VTS");
+	__ofono_modem_foreach_registered_atom(modem,
+						OFONO_ATOM_TYPE_EMULATOR_HFP,
+						emulator_remove_handler,
+						"D");
+	__ofono_modem_foreach_registered_atom(modem,
+						OFONO_ATOM_TYPE_EMULATOR_HFP,
+						emulator_remove_handler,
+						"+BLDN");
 
 	__ofono_modem_remove_atom_watch(modem, vc->hfp_watch);
 }
 
+static void voicecall_load_settings(struct ofono_voicecall *vc)
+{
+	const char *imsi;
+
+	imsi = ofono_sim_get_imsi(vc->sim);
+	if (imsi == NULL)
+		return;
+
+	vc->settings = storage_open(imsi, SETTINGS_STORE);
+
+	if (vc->settings == NULL)
+		return;
+
+	vc->imsi = g_strdup(imsi);
+}
+
+static void voicecall_close_settings(struct ofono_voicecall *vc)
+{
+	if (vc->settings) {
+		storage_close(vc->imsi, SETTINGS_STORE, vc->settings, TRUE);
+
+		g_free(vc->imsi);
+		vc->imsi = NULL;
+		vc->settings = NULL;
+	}
+}
+
 static void voicecall_unregister(struct ofono_atom *atom)
 {
 	DBusConnection *conn = ofono_dbus_get_connection();
@@ -2431,6 +2533,8 @@ static void voicecall_unregister(struct ofono_atom *atom)
 
 	emulator_hfp_unregister(atom);
 
+	voicecall_close_settings(vc);
+
 	if (vc->sim_state_watch) {
 		ofono_sim_remove_state_watch(vc->sim, vc->sim_state_watch);
 		vc->sim_state_watch = 0;
@@ -2570,6 +2674,9 @@ static void sim_state_watch(enum ofono_sim_state new_state, void *user)
 
 		free_sim_ecc_numbers(vc, FALSE);
 		set_new_ecc(vc);
+	case OFONO_SIM_STATE_READY:
+		voicecall_load_settings(vc);
+		break;
 	default:
 		break;
 	}
@@ -2582,6 +2689,7 @@ static void sim_watch(struct ofono_atom *atom,
 	struct ofono_sim *sim = __ofono_atom_get_data(atom);
 
 	if (cond == OFONO_ATOM_WATCH_CONDITION_UNREGISTERED) {
+		voicecall_close_settings(vc);
 		vc->sim_state_watch = 0;
 		vc->sim = NULL;
 		return;
@@ -2598,11 +2706,8 @@ static void sim_watch(struct ofono_atom *atom,
 static void emulator_generic_cb(const struct ofono_error *error, void *data)
 {
 	struct ofono_emulator *em = data;
-	struct ofono_error result;
 
-	result.type = error->type;
-
-	ofono_emulator_send_final(em, &result);
+	ofono_emulator_send_final(em, error);
 }
 
 static void emulator_ata_cb(struct ofono_emulator *em,
@@ -2736,6 +2841,335 @@ static void emulator_clcc_cb(struct ofono_emulator *em,
 	ofono_emulator_send_final(em, &result);
 }
 
+#define ADD_CHLD_SUPPORT(cond, x)			\
+	if (cond) {					\
+		if (info[-1] != '(')			\
+			*info++ = ',';			\
+							\
+		*info++ = x[0];				\
+							\
+		if (x[1])				\
+			*info++ = x[1];			\
+	}						\
+
+static void emulator_chld_cb(struct ofono_emulator *em,
+			struct ofono_emulator_request *req, void *userdata)
+{
+	struct ofono_voicecall *vc = userdata;
+	struct ofono_error result;
+	char buf[64];
+	char *info;
+	int chld;
+
+	result.error = 0;
+
+	switch (ofono_emulator_request_get_type(req)) {
+	case OFONO_EMULATOR_REQUEST_TYPE_SET:
+		if (!ofono_emulator_request_next_number(req, &chld))
+			goto fail;
+
+		switch (chld) {
+		case 0:
+			if (vc->driver->set_udub == NULL)
+				goto fail;
+
+			if (vc->driver->release_all_held == NULL)
+				goto fail;
+
+			if (voicecalls_have_waiting(vc)) {
+				vc->driver->set_udub(vc,
+						emulator_generic_cb, em);
+				return;
+			}
+
+			vc->driver->release_all_held(vc,
+					emulator_generic_cb, em);
+			return;
+		case 1:
+			if (vc->driver->release_all_active == NULL)
+				goto fail;
+
+			vc->driver->release_all_active(vc,
+					emulator_generic_cb, em);
+			return;
+		case 2:
+			if (vc->driver->hold_all_active == NULL)
+				goto fail;
+
+			vc->driver->hold_all_active(vc,
+					emulator_generic_cb, em);
+			return;
+		case 3:
+			if (vc->driver->create_multiparty == NULL)
+				goto fail;
+
+			vc->driver->create_multiparty(vc,
+					emulator_generic_cb, em);
+			return;
+		case 4:
+			if (vc->driver->transfer == NULL)
+				goto fail;
+
+			vc->driver->transfer(vc,
+					emulator_generic_cb, em);
+			return;
+		default:
+			break;
+		}
+
+		if (chld >= 11 && chld <= 17) {
+			if (vc->driver->release_specific == NULL)
+				goto fail;
+
+			vc->driver->release_specific(vc, chld - 10,
+					emulator_generic_cb, em);
+			return;
+		}
+
+		if (chld >= 21 && chld <= 27) {
+			if (vc->driver->private_chat == NULL)
+				goto fail;
+
+			vc->driver->private_chat(vc, chld - 20,
+					emulator_generic_cb, em);
+			return;
+		}
+
+		goto fail;
+
+	case OFONO_EMULATOR_REQUEST_TYPE_SUPPORT:
+		memcpy(buf, "+CHLD: (", 8);
+		info = buf + 8;
+
+		ADD_CHLD_SUPPORT(vc->driver->release_all_held &&
+					vc->driver->set_udub, "0")
+		ADD_CHLD_SUPPORT(vc->driver->release_all_active, "1")
+		ADD_CHLD_SUPPORT(vc->driver->release_specific, "1x")
+		ADD_CHLD_SUPPORT(vc->driver->hold_all_active, "2")
+		ADD_CHLD_SUPPORT(vc->driver->private_chat, "2x")
+		ADD_CHLD_SUPPORT(vc->driver->create_multiparty, "3")
+		ADD_CHLD_SUPPORT(vc->driver->transfer, "4")
+
+		*info++ = ')';
+		*info++ = '\0';
+
+		ofono_emulator_send_info(em, buf, TRUE);
+		result.type = OFONO_ERROR_TYPE_NO_ERROR;
+		break;
+
+	case OFONO_EMULATOR_REQUEST_TYPE_QUERY:
+	case OFONO_EMULATOR_REQUEST_TYPE_COMMAND_ONLY:
+fail:
+		result.type = OFONO_ERROR_TYPE_FAILURE;
+	}
+
+	ofono_emulator_send_final(em, &result);
+}
+
+static void vts_tone_cb(int error, void *data)
+{
+	struct ofono_emulator *em = data;
+	struct ofono_error result;
+
+	result.error = 0;
+	result.type = error ? OFONO_ERROR_TYPE_FAILURE :
+						OFONO_ERROR_TYPE_NO_ERROR;
+
+	ofono_emulator_send_final(em, &result);
+}
+
+static void emulator_vts_cb(struct ofono_emulator *em,
+			struct ofono_emulator_request *req, void *userdata)
+{
+	struct ofono_voicecall *vc = userdata;
+	struct ofono_error result;
+	const char *str;
+
+	switch (ofono_emulator_request_get_type(req)) {
+	case OFONO_EMULATOR_REQUEST_TYPE_SET:
+		str = ofono_emulator_request_get_raw(req);
+		if (str == NULL)
+			break;
+
+		if (!g_ascii_isdigit(str[0]) && str[0] != '*' && str[0] != '#'
+				&& (str[0] < 'A' || str[0] > 'D'))
+			break;
+
+		if (str[1] != '\0')
+			break;
+
+		if (__ofono_voicecall_tone_send(vc, str, vts_tone_cb, em) >= 0)
+			return;
+
+		break;
+
+	default:
+		break;
+	}
+
+	result.error = 0;
+	result.type = OFONO_ERROR_TYPE_FAILURE;
+
+	ofono_emulator_send_final(em, &result);
+}
+
+static void emulator_dial_callback(const struct ofono_error *error, void *data)
+{
+	struct ofono_voicecall *vc = data;
+	gboolean need_to_emit;
+	struct voicecall *v;
+
+	v = dial_handle_result(vc, error, vc->em_atd_number, &need_to_emit);
+
+	if (v == NULL) {
+		struct ofono_modem *modem = __ofono_atom_get_modem(vc->atom);
+
+		if (is_emergency_number(vc, vc->em_atd_number) == TRUE)
+			__ofono_modem_dec_emergency_mode(modem);
+	}
+
+	ofono_emulator_send_final(vc->pending_em, error);
+
+	vc->pending_em = NULL;
+	g_free(vc->em_atd_number);
+	vc->em_atd_number = NULL;
+
+	notify_emulator_call_status(vc);
+
+	if (need_to_emit)
+		voicecalls_emit_call_added(vc, v);
+}
+
+static void emulator_dial(struct ofono_emulator *em, struct ofono_voicecall *vc,
+				const char *number)
+{
+	struct ofono_error result;
+	int err;
+
+	result.error = 0;
+
+	if (vc->pending || vc->dial_req || vc->pending_em) {
+		result.type = OFONO_ERROR_TYPE_FAILURE;
+		goto send;
+	}
+
+	vc->pending_em = em;
+	vc->em_atd_number = g_strdup(number);
+
+	err = voicecall_dial(vc, number, OFONO_CLIR_OPTION_DEFAULT,
+					emulator_dial_callback, vc);
+
+	if (err >= 0)
+		return;
+
+	vc->pending_em = NULL;
+	g_free(vc->em_atd_number);
+	vc->em_atd_number = NULL;
+
+	switch (err) {
+	case -ENETDOWN:
+		result.error = 30;
+		result.type = OFONO_ERROR_TYPE_CME;
+		break;
+
+	default:
+		result.type = OFONO_ERROR_TYPE_FAILURE;
+	}
+
+send:
+	ofono_emulator_send_final(em, &result);
+}
+
+static void emulator_atd_cb(struct ofono_emulator *em,
+			struct ofono_emulator_request *req, void *userdata)
+{
+	struct ofono_voicecall *vc = userdata;
+	struct ofono_modem *modem = __ofono_atom_get_modem(vc->atom);
+	const char *str;
+	size_t len;
+	char number[OFONO_MAX_PHONE_NUMBER_LENGTH + 1];
+	struct ofono_error result;
+
+	switch (ofono_emulator_request_get_type(req)) {
+	case OFONO_EMULATOR_REQUEST_TYPE_SET:
+		str = ofono_emulator_request_get_raw(req);
+
+		if (str == NULL || str[0] == '\0')
+			goto fail;
+
+		len = strlen(str);
+
+		if (len > OFONO_MAX_PHONE_NUMBER_LENGTH + 1 ||
+				str[len - 1] != ';')
+			goto fail;
+
+		if (len == 3 && str[0] == '>' && str[1] == '1') {
+			struct ofono_atom *mw_atom;
+			struct ofono_message_waiting *mw;
+			const struct ofono_phone_number *ph;
+			const char *num;
+
+			mw_atom = __ofono_modem_find_atom(modem,
+					OFONO_ATOM_TYPE_MESSAGE_WAITING);
+
+			if (mw_atom == NULL)
+				goto fail;
+
+			mw = __ofono_atom_get_data(mw_atom);
+			ph = __ofono_message_waiting_get_mbdn(mw, 0);
+
+			if (ph == NULL)
+				goto fail;
+
+			num = phone_number_to_string(ph);
+
+			emulator_dial(em, vc, num);
+		} else {
+			strncpy(number, str, len - 1);
+			number[len - 1] = '\0';
+
+			emulator_dial(em, vc, number);
+		}
+
+		break;
+
+	default:
+fail:
+		result.error = 0;
+		result.type = OFONO_ERROR_TYPE_FAILURE;
+		ofono_emulator_send_final(em, &result);
+	};
+}
+
+static void emulator_bldn_cb(struct ofono_emulator *em,
+			struct ofono_emulator_request *req, void *userdata)
+{
+	struct ofono_voicecall *vc = userdata;
+	const char *number;
+	struct ofono_error result;
+	GError *error = NULL;
+
+	switch (ofono_emulator_request_get_type(req)) {
+	case OFONO_EMULATOR_REQUEST_TYPE_COMMAND_ONLY:
+		if (vc->settings == NULL)
+			goto fail;
+
+		number = g_key_file_get_string(vc->settings, SETTINGS_GROUP,
+						"Number", &error);
+		if (number == NULL || number[0] == '\0')
+			goto fail;
+
+		emulator_dial(em, vc, number);
+		break;
+
+	default:
+fail:
+		result.error = 0;
+		result.type = OFONO_ERROR_TYPE_FAILURE;
+		ofono_emulator_send_final(em, &result);
+	};
+}
+
 static void emulator_hfp_watch(struct ofono_atom *atom,
 				enum ofono_atom_watch_condition cond,
 				void *data)
@@ -2750,6 +3184,10 @@ static void emulator_hfp_watch(struct ofono_atom *atom,
 	ofono_emulator_add_handler(em, "A", emulator_ata_cb, data, NULL);
 	ofono_emulator_add_handler(em, "+CHUP", emulator_chup_cb, data, NULL);
 	ofono_emulator_add_handler(em, "+CLCC", emulator_clcc_cb, data, NULL);
+	ofono_emulator_add_handler(em, "+CHLD", emulator_chld_cb, data, NULL);
+	ofono_emulator_add_handler(em, "+VTS", emulator_vts_cb, data, NULL);
+	ofono_emulator_add_handler(em, "D", emulator_atd_cb, data, NULL);
+	ofono_emulator_add_handler(em, "+BLDN", emulator_bldn_cb, data, NULL);
 }
 
 void ofono_voicecall_register(struct ofono_voicecall *vc)
@@ -2820,7 +3258,7 @@ int ofono_voicecall_get_next_callid(struct ofono_voicecall *vc)
 ofono_bool_t __ofono_voicecall_is_busy(struct ofono_voicecall *vc,
 					enum ofono_voicecall_interaction type)
 {
-	if (vc->pending || vc->dial_req)
+	if (vc->pending || vc->dial_req || vc->pending_em)
 		return TRUE;
 
 	switch (type) {
